OK = 0
UNKNOWN = 255

CONFIG_INITIALIZATION_ERROR = 1
INVALID_SETTINGS = 2
MODEL_NOT_DEFINED = 3
PHASE_NOT_DEFINED = 4
INPUT_TYPE_NOT_DEFINED = 5
INVALID_INPUT_TYPE = 6

MODEL_CODE_IMPORT_ERR = 7
CONFIG_NOT_INITIALIZED = 8
INVALID_CONFIG_PATH = 9
MODEL_DOWNLOADING_ERROR = 10
INVALID_PHASE = 11
CALLBACK_IS_NOT_CALLABLE = 12
INPUT_TOPIC_NOT_DEFINED = 13